/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import java.lang.reflect.InvocationTargetException;
import java.util.Optional;

import org.apache.commons.lang3.reflect.ConstructorUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import de.ozgcloud.common.errorhandling.TechnicalException;

@Configuration
public class AktenzeichenProviderConfiguration {

	private static final String MSG_CLASSNOTFOUND_TEMPL = "Class '%s' for AktenzeichenProvider not found";
	private static final String MSG_INSTANCE = "Error on getInstance for aktenzeichen provider.";

	// TODO configruations-hierachie verwenden
	@Value("${ozgcloud.aktenzeichen:#{null}}")
	private Optional<String> aktenzeichenCalculator;

	@Bean
	public AktenzeichenProvider provideAktenzeichenCalulator() {
		return aktenzeichenCalculator.map(this::getProvider).orElse(getDefaultProvider());
	}

	AktenzeichenProvider getDefaultProvider() {
		return new DefaultAktenzeichenProvider();
	}

	@SuppressWarnings("unchecked")
	private AktenzeichenProvider getProvider(String className) {
		try {
			Class<AktenzeichenProvider> clazz = (Class<AktenzeichenProvider>) Class.forName(className);
			return ConstructorUtils.invokeConstructor(clazz);
		} catch (ClassNotFoundException e) {
			throw new TechnicalException(String.format(MSG_CLASSNOTFOUND_TEMPL, className), e);
		} catch (IllegalAccessException | InstantiationException | InvocationTargetException | NoSuchMethodException e) {
			throw new TechnicalException(MSG_INSTANCE, e);
		}
	}
}