package de.ozgcloud.vorgang.common.migration;

import static org.assertj.core.api.Assertions.*;

import org.bson.Document;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Query;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.common.test.DataITCase;

@DataITCase
class M011_SetDefaultCollaborationLevelITCase {

	private final M011_SetDefaultCollaborationLevel migration = new M011_SetDefaultCollaborationLevel();

	private final String nameField = "name";
	private final String nameFieldValue = LoremIpsum.getInstance().getWords(1);

	private final String organisationsEinheitField = "organisationsEinheitId";
	private final String organisationsEinheitFieldValue = LoremIpsum.getInstance().getWords(1);

	@Autowired
	private MongoTemplate template;
	@Autowired
	private MigrationDbTestUtils dbTestUtils;

	@DisplayName("Do migration")
	@Nested
	class TestDoMigration {

		@BeforeEach
		void mock() {
			dbTestUtils.dropVorgangCollection();
		}

		@DisplayName("on missing collaboration level")
		@Nested
		class TestOnMissingCollaborationLevel {

			@BeforeEach
			void initVorgang() {
				dbTestUtils.saveVorgang(createVorgangWithName());
			}

			@Test
			void shouldSetDefaultCollaborationLevel() {
				migration.doMigration(template);

				assertThat(getVorgangHeader()).containsEntry(M011_SetDefaultCollaborationLevel.COLLABORATION_LEVEL_FIELD, 0);
			}

			@Test
			void shouldKeepVorgangData() {
				migration.doMigration(template);

				assertThat(getVorgang()).containsEntry(nameField, nameFieldValue);
			}

			@Test
			void shouldKeepHeaderData() {
				migration.doMigration(template);

				assertThat(getVorgangHeader()).containsEntry(organisationsEinheitField,
						organisationsEinheitFieldValue);
			}
		}

		@DisplayName("on existing collaboration level")
		@Nested
		class TestOnExistingCollaborationLevel {

			@BeforeEach
			void initVorgang() {
				dbTestUtils.saveVorgang(createVorgangWithCollaborationLevel());
			}

			@Test
			void shouldKeepCollaborationLevel() {
				migration.doMigration(template);

				assertThat(getVorgangHeader()).containsEntry(M011_SetDefaultCollaborationLevel.COLLABORATION_LEVEL_FIELD, 1);
			}

			@Test
			void shouldKeepVorgangData() {
				migration.doMigration(template);

				assertThat(getVorgang()).containsEntry(nameField, nameFieldValue);
			}

			@Test
			void shouldKeepHeaderData() {
				migration.doMigration(template);

				assertThat(getVorgangHeader()).containsEntry(organisationsEinheitField,
						organisationsEinheitFieldValue);
			}
		}

		private Document createVorgangWithCollaborationLevel() {
			var vorgang = createVorgangWithName();
			vorgang.append(M011_SetDefaultCollaborationLevel.HEADER_FIELD, createHeaderWithCollaborationLevel());
			return vorgang;
		}

		private Document createHeaderWithCollaborationLevel() {
			var header = createHeaderWithOrganisationsEinheitId();
			header.append(M011_SetDefaultCollaborationLevel.COLLABORATION_LEVEL_FIELD, 1);
			return header;
		}

		private Document createVorgangWithName() {
			var vorgang = new Document();
			vorgang.append(nameField, nameFieldValue);
			vorgang.append(M011_SetDefaultCollaborationLevel.HEADER_FIELD, createHeaderWithOrganisationsEinheitId());
			return vorgang;
		}

		private Document createHeaderWithOrganisationsEinheitId() {
			var header = new Document();
			header.append(organisationsEinheitField, organisationsEinheitFieldValue);
			return header;
		}

		private Document getVorgangHeader() {
			return (Document) getVorgang().get(M011_SetDefaultCollaborationLevel.HEADER_FIELD);
		}

		private Document getVorgang() {
			var vorgaenge = dbTestUtils.findVorgang(new Query());
			assertThat(vorgaenge).hasSize(1);
			return vorgaenge.get(0);
		}
	}
}