/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.vorgang.servicekonto.GrpcServiceKontoTestFactory;
import de.ozgcloud.vorgang.servicekonto.ServiceKontoMapper;
import de.ozgcloud.vorgang.servicekonto.ServiceKontoTestFactory;

@DisplayName("Eingang header mapper")
class EingangHeaderMapperTest {

	@InjectMocks
	private final EingangHeaderMapper mapper = Mappers.getMapper(EingangHeaderMapper.class);
	@Mock
	private final ServiceKontoMapper serviceKontoMapper = Mappers.getMapper(ServiceKontoMapper.class);

	@DisplayName("map from grpc")
	@Nested
	class TestFromGrpc {

		@BeforeEach
		void mockPostfachAddressMapper() {
			when(serviceKontoMapper.fromServiceKonto(any())).thenReturn(ServiceKontoTestFactory.create());
		}

		@Test
		void shouldCallPostfachAddressMapper() {
			fromGrpc();

			verify(serviceKontoMapper).fromServiceKonto(any());
		}

		@Test
		void shouldMapDateStrToZonedDateTime() {
			var eingangHeader = fromGrpc();

			assertThat(eingangHeader.getCreatedAt()).isEqualTo(GrpcEingangHeaderTestFactory.CREATED_AT);
		}

		@Test
		void shouldMapAllFields() {
			var eingangHeader = fromGrpc();

			assertThat(eingangHeader).usingRecursiveComparison().ignoringFields("serviceKonto").isEqualTo(EingangHeaderTestFactory.create());
		}

		private EingangHeader fromGrpc() {
			return mapper.fromGrpc(GrpcEingangHeaderTestFactory.create());
		}
	}

	@DisplayName("map to grpc")
	@Nested
	class TestToGrpc {

		@BeforeEach
		void mockPostfachAddressMapper() {
			when(serviceKontoMapper.toServiceKonto(any())).thenReturn(GrpcServiceKontoTestFactory.create());
		}

		@Test
		void shouldMapZonedDateTimeToDateStr() {
			var eingangHeader = fromGrpc(EingangHeaderTestFactory.create());

			assertThat(eingangHeader.getCreatedAt()).isEqualTo(EingangHeaderTestFactory.CREATED_AT_STR);
		}

		@Test
		void shouldProceedWithEmptyValues() {
			var eingangHeader = fromGrpc(EingangHeaderTestFactory.createBuilder().createdAt(null).build());

			assertThat(eingangHeader.getCreatedAt()).isEqualTo(StringUtils.EMPTY);
		}

		private GrpcEingangHeader fromGrpc(EingangHeader eingangHeader) {
			return mapper.toGrpc(eingangHeader);
		}
	}
}